# -*- coding: utf-8 -*-
###############################################################################
#
#   Module for OpenERP 
#   Copyright (C) 2013 Akretion (http://www.akretion.com).
#   @author Sébastien BEAU <sebastien.beau@akretion.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################



import unittest2
from functools import partial

from openerp.addons.connector.exception import InvalidDataError
from openerp.addons.prestashoperpconnect.unit.import_synchronizer import (
            import_batch,
            import_record)
from openerp.addons.connector.session import ConnectorSession
import openerp.tests.common as common
from .common import (mock_api)
from .test_data import prestashop_base_responses
from openerp.addons.prestashoperpconnect.connector import get_environment
from openerp.addons.prestashoperpconnect.unit.direct_binder import DirectBinder

DB = common.DB
ADMIN_USER_ID = common.ADMIN_USER_ID



class test_import_prestashop(common.SingleTransactionCase):
    """ Test the imports from a Prestashop Mock.

    The data returned by Prestashop are those created for the
    demo version of Prestashop on a standard 1.7 version.
    """

    def setUp(self):
        super(test_import_prestashop, self).setUp()
        self.backend_model = self.registry('prestashop.backend')
        self.session = ConnectorSession(self.cr, self.uid)
        data_model = self.registry('ir.model.data')
        self.get_ref = partial(data_model.get_object_reference,
                               self.cr, self.uid)
        backend_ids = self.backend_model.search(
            self.cr, self.uid,
            [('name', '=', 'Test Prestashop')])
        if backend_ids:
            self.backend_id = backend_ids[0]
        else:
            __, warehouse_id = self.get_ref('stock', 'warehouse0')
            self.backend_id = self.backend_model.create(
                self.cr,
                self.uid,
                {'name': 'Test Prestashop',
                 'version': '1.5',
                 'location': 'http://anyurl',
                 'username': 'sebastien',
                 'warehouse_id': warehouse_id,
                 'password': '42'})


    def test_00_import_backend(self):
        """ Synchronize initial metadata """
        with mock_api(prestashop_base_responses):
            import_batch(self.session, 'prestashop.shop.group', self.backend_id)
            import_batch(self.session, 'prestashop.shop', self.backend_id)

        shop_group_model = self.registry('prestashop.shop.group')
        shop_group_ids = shop_group_model.search(self.cr,
                                           self.uid,
                                           [('backend_id', '=', self.backend_id)])
        self.assertEqual(len(shop_group_ids), 1)

        shop_model = self.registry('prestashop.shop')
        shop_ids = shop_model.search(self.cr,
                                       self.uid,
                                       [('backend_id', '=', self.backend_id)])
        self.assertEqual(len(shop_ids), 1)

    def test_01_map_lang(self):
        """ Map language """
        with mock_api(prestashop_base_responses):
            env = get_environment(self.session, 'prestashop.res.lang', self.backend_id)
            directBinder = env.get_connector_unit(DirectBinder)
            directBinder.run()
        lang_model = self.registry('prestashop.res.lang')
        lang_ids = lang_model.search(self.cr,
                                       self.uid,
                                       [('backend_id', '=', self.backend_id)])
        self.assertEqual(len(lang_ids), 1)

    def test_10_import_product_category(self):
        """ Import of a product category """
        backend_id = self.backend_id
        with mock_api(prestashop_base_responses):
            import_record(self.session, 'prestashop.product.category',
                          backend_id, 1)

        category_model = self.registry('prestashop.product.category')
        category_ids = category_model.search(
            self.cr, self.uid, [('backend_id', '=', backend_id)])
        self.assertEqual(len(category_ids), 1)

    def test_11_import_product_category_with_gap(self):
        """ Import of a product category when parent categories are missing """
        backend_id = self.backend_id
        with mock_api(prestashop_base_responses):
            import_record(self.session, 'prestashop.product.category',
                          backend_id, 3)

        category_model = self.registry('prestashop.product.category')
        category_ids = category_model.search(
            self.cr, self.uid, [('backend_id', '=', backend_id)])
        self.assertEqual(len(category_ids), 3)


